const {Type} = require('xcraft-core-stones');

/**
 * @template T
 * @typedef {import("xcraft-core-stones").t<T>} t
 */
/**
 * @typedef {import("xcraft-core-stones").AnyTypeOrShape} AnyTypeOrShape
 */

/**
 * @template {string} T
 * @extends {Type<`${T}@{string}`>}
 */
class IdType extends Type {
  /** @param {T} name */
  constructor(name) {
    super('id');
    this.name = name;
  }

  get fullName() {
    return `\`${this.name}@{string}\``;
  }

  /** @type {Type["check"]} */
  check(value, check) {
    if (!check.typeOf(value, 'string')) {
      return;
    }
    check.true(value.startsWith(this.name + '@'), 'bad pattern', {
      actual: value,
      expectedValues: this.fullName,
    });
  }
}

/** @type {<const T extends string>(name: T) => Type<`${T}@${string}`>} */
const id = (name) => new IdType(name);

module.exports = {
  IdType,
  id,
};
